/***
 * REXX Communication Control Function Library  Ver.1.9
 *                  copyright 1998-2002, Naga(Kazuo Adachi)
 *                          mailto:nagaw@attglobal.net
 *
 * $Id$
 *
 * $Log$
 */

#include "RxComm.h"
#include <malloc.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>


/*--------------------------------------------------------*/
/**
 * REXX Communication Variables.
 */
RXCOMM      rxcInfo;

/*--------------------------------------------------------*/
/**
 * REXX Communication Function List.
 */
static PSZ  RxFncTable[] =
{
    "RxCommLoadFuncs",
    "RxCommDropFuncs",
    "RxCommGetVersion",
    "RxCommPort",
    "RxCommOpen",
    "RxCommClose",
    "RxCommRead",
    "RxCommWrite",
    "RxCommSignal",
    "RxCommBreak",
    "RxCommSetup",
    "RxCommOption",
    "RxCommLog",

};


/*--------------------------------------------------------*/
/*--------------------------------------------------------*/
/**
 * REXX Communication Load Function.
 */
ULONG APIENTRY RxCommLoadFuncs(PSZ name,ULONG argc,RXSTRING argv[],PSZ queue,PRXSTRING retstr)
{
    INT entries;
    INT cnIndex;

    retstr->strlength = 0;
    if ( 0 < argc )
    {
        return 40;
    }

    /**
     * Register Communication Function Entries.
     */
    entries = sizeof(RxFncTable) / sizeof(PSZ);
    for ( cnIndex = 0; cnIndex < entries; cnIndex++ )
    {
        RexxRegisterFunctionDll( RxFncTable[cnIndex], "RXCOMM", RxFncTable[cnIndex] );
    }

    return 0;
}

/**
 * REXX Communication Drop Function.
 */
ULONG APIENTRY RxCommDropFuncs(PSZ name,ULONG argc,RXSTRING argv[],PSZ queue,PRXSTRING retstr)
{
    INT entries;
    INT cnIndex;

    retstr->strlength = 0;
    if ( 0 < argc )
    {
        return 40;
    }

    /**
     * Deregister Communication Function Entries.
     */
    entries = sizeof(RxFncTable) / sizeof(PSZ);
    for ( cnIndex = 0; cnIndex < entries; cnIndex++ )
    {
        RexxDeregisterFunction( RxFncTable[cnIndex] );
    }

    return 0;
}


/*--------------------------------------------------------*/
/*--------------------------------------------------------*/
/**
 * REXX Communication Get Version.
 */
ULONG APIENTRY RxCommGetVersion(PSZ name,ULONG argc,RXSTRING argv[],PSZ queue,PRXSTRING retstr)
{
    retstr->strlength = 0;
    if ( 0 < argc )
    {
        return 40;
    }

    /**
     * Return Version Number of this Library.
     */
    strcpy( retstr->strptr, RXCOMM_REVISION );
    retstr->strlength = strlen( retstr->strptr );

    return 0;
}

/*--------------------------------------------------------*/
/**
 * REXX Communication Get Port Count Function.
 */
ULONG APIENTRY RxCommPort(PSZ name,ULONG argc,RXSTRING argv[],PSZ queue,PRXSTRING retstr)
{
    APIRET retCode;
    BYTE   nRS232C;
    HFILE  hcomFile;
    ULONG  ulAction;

    retstr->strlength = 0;
    if ( 1 < argc )
    {
        return 40;
    }

    if ( (0 == argc) || (0 == RXSTRLEN(argv[0])) )
    {
        /**
         * Return Numbers of COM Port to have.
         */
        retCode = DosDevConfig( &nRS232C, DEVINFO_RS232 );
        if ( NO_ERROR != retCode )
        {
            nRS232C = 0;
        }
    }
    else
    {
        /**
         * Check to be able to Open COM Port.
         */
        nRS232C = 0;
        retCode = DosOpen( RXSTRPTR(argv[0]), &hcomFile, &ulAction, 0L,
            FILE_NORMAL, FILE_OPEN, OPEN_SHARE_DENYNONE, 0L );
        if ( NO_ERROR == retCode )
        {
            nRS232C = 1;
            DosClose( hcomFile );
        }
    }

    sprintf( retstr->strptr, "%d", nRS232C );
    retstr->strlength = strlen( retstr->strptr );

    return 0;
}


/*--------------------------------------------------------*/
/*--------------------------------------------------------*/
/**
 * REXX Communication DLL Init/Term.
 */
int  _CRT_init( void );
void _CRT_term( void );

ULONG _System _DLL_InitTerm( ULONG hModule, ULONG ulFlag )
{
    switch ( ulFlag )
    {
        case 0:
            if ( -1 == _CRT_init() )
            {
                return 0UL;
            }

            /**
             * Initialyze Communication Global Variables.
             */
            rxcInfo.pszName = NULL;
            rxcInfo.hcmFile = NULLHANDLE;
            rxcInfo.nulStrp = TRUE;
            rxcInfo.sndChar = FALSE;
            rxcInfo.rcvChar = FALSE;
            rxcInfo.logMode = FALSE;
            rxcInfo.binMode = FALSE;
            rxcInfo.sndRxCR = RXCH_CR;
            rxcInfo.sndRxLF = RXCH_NULL;
            rxcInfo.rcvRxCR = RXCH_ORG;
            rxcInfo.rcvRxLF = RXCH_ORG;

            rxcInfo.pszRecv = NULL;
            rxcInfo.maxRecv = DEF_RECVBUFF;
            rxcInfo.posRecv = 0;

            rxcInfo.pszSend = NULL;
            rxcInfo.maxSend = DEF_SENDBUFF;

            rxcInfo.logFile = NULLHANDLE;

            break;
        case 1:
            /**
             * Terminate Communication Global Variables.
             */
            if ( NULLHANDLE != rxcInfo.logFile )
            {
                DosClose( rxcInfo.logFile );
                rxcInfo.logFile = NULLHANDLE;
            }

            if ( NULLHANDLE != rxcInfo.hcmFile )
            {
                DosClose( rxcInfo.hcmFile );
                rxcInfo.hcmFile = NULLHANDLE;
            }

            if ( NULL != rxcInfo.pszSend )
            {
                free( rxcInfo.pszSend );
                rxcInfo.pszSend = NULL;
            }
            if ( NULL != rxcInfo.pszRecv )
            {
                free( rxcInfo.pszRecv );
                rxcInfo.pszRecv = NULL;
            }

            _CRT_term();

            break;
        default:
            return 0UL;
    }

    return 1UL;
}

