/***
 * REXX Communication Control Function Library  Ver.1.9
 *                  copyright 1998-2002, Naga(Kazuo Adachi)
 *                          mailto:nagaw@attglobal.net
 *
 * $Id$
 *
 * $Log$
 */

#include "RxComm.h"
#include <ctype.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>


/*--------------------------------------------------------*/
/**
 * REXX Communication Write Function.
 */
static CHAR _htoi( CHAR chHex )
{
    static const PSZ pHex = "00112233445566778899AaBbCcDdEeFf";

    return ((PSZ)strchr( pHex, chHex ) - pHex) / 2;
}

/*--------------------------------------------------------*/
/*--------------------------------------------------------*/
/**
 * REXX Communication Write Function.
 */
ULONG APIENTRY RxCommWrite(PSZ name,ULONG argc,RXSTRING argv[],PSZ queue,PRXSTRING retstr)
{
    APIRET retCode;
    HFILE  hcomFile;
    PSZ    psText;
    PSZ    psBuff;
    CHAR   chSend;
    RXCRLF rxCRLF;
    ULONG  ulText;
    ULONG  ulBuff;
    ULONG  ulSize;
    ULONG  ulSend;
    ULONG  ulTry;

    retstr->strlength = 0;
    if ( 1 != argc )
    {
        return 40;
    }
    if ( NULLHANDLE == rxcInfo.hcmFile )
    {
        return 40;
    }

    hcomFile = rxcInfo.hcmFile;

    /**
     * convert escape control char.
     */
    psText = RXSTRPTR(argv[0]);
    ulText = RXSTRLEN(argv[0]);
    if ( rxcInfo.maxSend < ulText )
    {
        free( rxcInfo.pszSend );
        rxcInfo.pszSend = malloc( ulText );
        rxcInfo.maxSend = ulText;
    }
    psBuff = rxcInfo.pszSend;
    for ( ulBuff = 0, ulSize = 0; ulSize < ulText; ulSize++ )
    {
        if ( rxcInfo.sndChar && ('\\' == psText[ulSize]) )
        {
            ulSize++;
            switch ( psText[ulSize] )
            {
                case '0':
                case '1':
                case '2':
                case '3':
                case '4':
                case '5':
                case '6':
                case '7':
                    chSend = psText[ulSize] - '0';
                    if ( '0' <= psText[ulSize+1] && psText[ulSize+1] <= '7' )
                    {
                        chSend *= 8;
                        chSend += psText[ulSize+1] - '0';
                        if ( '0' <= psText[ulSize+2] && psText[ulSize+2] <= '7' )
                        {
                            chSend *= 8;
                            chSend += psText[ulSize+2] - '0';
                            ulSize++;
                        }
                        ulSize++;
                    }
                    break;
                case 'a':  chSend = 0x07; break;
                case 'b':  chSend = 0x08; break;
                case 'e':  chSend = 0x1b; break;
                case 'f':  chSend = 0x0c; break;
                case 'n':  chSend = 0x0a; break;
                case 'r':  chSend = 0x0d; break;
                case 't':  chSend = 0x09; break;
                case 'v':  chSend = 0x0b; break;
                case '\\': chSend = 0x5c; break;
                case 'x':
                    if ( isxdigit( psText[ulSize+1] ) )
                    {
                        chSend = _htoi( psText[ulSize+1] );
                        if ( isxdigit( psText[ulSize+2] ) )
                        {
                            chSend *= 0x10;
                            chSend += _htoi( psText[ulSize+2] );
                            ulSize++;
                        }
                        ulSize++;
                        break;
                    }
                default:
                    return 40;
            }
        }
        else
        {
            chSend = psText[ulSize];
        }

        rxCRLF = RXCH_ORG;
        if ( '\x0d' == chSend )
        {
            rxCRLF = rxcInfo.sndRxCR;
        }
        else
        if ( '\x0a' == chSend )
        {
            rxCRLF = rxcInfo.sndRxLF;
        }
        switch ( rxCRLF )
        {
            case RXCH_NULL:
                break;
            case RXCH_CR:
                psBuff[ulBuff++] = 0x0d;
                break;
            case RXCH_LF:
                psBuff[ulBuff++] = 0x0a;
                break;
            case RXCH_CRLF:
                psBuff[ulBuff++] = 0x0d;
                psBuff[ulBuff++] = 0x0a;
                break;
            case RXCH_ORG:
                psBuff[ulBuff++] = chSend;
                break;
        }
    }

    ulSend = 0;

    /**
     * write(send) string to COM device
     */
    for ( ulTry = 0; (ulTry < DEF_WRITETRY) && (ulSend < ulBuff); ulTry++ )
    {
        DosEnterCritSec();
        retCode = DosWrite( hcomFile, &psBuff[ulSend], ulBuff-ulSend, &ulSize );
        DosExitCritSec();
        if ( NO_ERROR == retCode )
        {
            ulSend += ulSize;
        }
    }

    sprintf( retstr->strptr, "%d", ulSend );
    retstr->strlength = strlen( retstr->strptr );

    return 0;
}

